//
//  ControllerView.swift
//  Pomelo-V2
//
//  Created by Stossy11 on 16/7/2024.
//

import SwiftUI
import GameController
import SwiftUIJoystick
import CoreMotion

struct ControllerView: View {
    var body: some View {
        GeometryReader { geometry in
            if geometry.size.height > geometry.size.width && UIDevice.current.userInterfaceIdiom != .pad {
                VStack {
                    
                    Spacer()
                    VStack {
                        HStack {
                            VStack {
                                ShoulderButtonsViewLeft()
                                ZStack {
                                    Joystick()
                                    DPadView()
                                }
                            }
                            Spacer()
                            VStack {
                                ShoulderButtonsViewRight()
                                ZStack {
                                    Joystick(iscool: true) // hope this works JoyStick
                                    ABXYView()
                                }
                            }
                        }
                        
                        HStack {
                            ButtonView(button: .start) // Adding the + button
                                .padding(.horizontal, 40)
                            ButtonView(button: .back) // Adding the - button
                                .padding(.horizontal, 40)
                        }
                    }
                }
                
            } else {
                // could be landscape
                /*VStack {
                    
                    Spacer()
                    VStack {
                        HStack {
                            
                            // gotta fuckin add + and - now
                            VStack {
                                ShoulderButtonsViewLeft()
                                ZStack {
                                    //Joystick()
                                    Joystick4FTG()
                                    //DPadView() Disable landscape Dpad
                                }
                            }
                            HStack {
                                // Spacer()
                                VStack {
                                    // Spacer()
                                    ButtonView(button: .back) // Adding the - button
                                }
                                Spacer()
                                VStack {
                                    // Spacer()
                                    ButtonView(button: .start) // Adding the + button
                                }
                                // Spacer()
                            }
                            VStack {
                                ShoulderButtonsViewRight()
                                ZStack {
                                    //Joystick(iscool: true) // hope this work Disable right Joystick
                                    ABXYView()
                                }
                            }
                        }
                        
                    }
                    // .padding(.bottom, geometry.size.height / 11) // also extremally broken (
                }*/
               
                    VStack {
                        Spacer() // 占据顶部空间
                        HStack(alignment: .bottom) {
                            VStack(alignment: .leading) {
                                //ShoulderButtonsViewLeft()
                                Joystick4FTG()  // 摇杆控件
                                //.frame(maxWidth: .infinity, alignment: .leading)
                            }
                            .offset(x: 10, y: -5) // 向右移动（+） ，向上移动（﹣）
                            HStack {
                                VStack {
                                    ButtonView(button: .back) // - 按钮
                                }
                                Spacer()
                                VStack {
                                    ButtonView(button: .start) // + 按钮
                                }
                            }
                            
                            VStack {
                                ShoulderButtonsViewRight()
                                ZStack {
                                    ABXYView()
                                }
                            }.offset(x: 30, y: 25) // 向右移动 50，向下移动 25
                        }
                    .padding(.bottom, 10) // 适当增加 padding 让它靠近底部
                }
            }
        }
        .padding()
    }
}

struct ShoulderButtonsViewLeft: View {
    @State var width: CGFloat = 160
    @State var height: CGFloat = 20
    @AppStorage("On-ScreenControllerScale") var controllerScale: Double = 1.0
    
    var body: some View {
        HStack {
            //move left Trigger
           // ButtonView(button: .leftTrigger)
             //   .padding(.horizontal)
            //ButtonView(button: .leftShoulder)
             //   .padding(.horizontal)
        }
        .frame(width: width, height: height)
        .onAppear() {
            if UIDevice.current.systemName.contains("iPadOS") {
                width *= 1.2
                height *= 1.2
            }
            
            width *= CGFloat(controllerScale)
            height *= CGFloat(controllerScale)
        }
    }
}

struct ShoulderButtonsViewRight: View {
    @State var width: CGFloat = 100
    @State var height: CGFloat = 60
    @AppStorage("On-ScreenControllerScale") var controllerScale: Double = 1.0
    
    var body: some View {
        HStack {
            ButtonView(button: .rightShoulder)
                .padding(.horizontal)
            ButtonView(button: .rightTrigger)
                .padding(.horizontal)
           
        }
        HStack{
            //put left Trigger there
            ButtonView(button: .leftTrigger)
                .padding(.horizontal)
            ButtonView(button: .leftShoulder)
                .padding(.horizontal)
            
        }
        .frame(width: width, height: height)
        .onAppear() {
            if UIDevice.current.systemName.contains("iPadOS") {
                width *= 1.2
                height *= 1.2
            }
            
            width *= CGFloat(controllerScale)
            height *= CGFloat(controllerScale)
        }
    }
}

struct DPadView: View {
    @State var size: CGFloat = 145
    @AppStorage("On-ScreenControllerScale") var controllerScale: Double = 1.0
    var body: some View {
        VStack {
            ButtonView(button: .dPadUp)
            HStack {
                ButtonView(button: .dPadLeft)
                Spacer(minLength: 20)
                ButtonView(button: .dPadRight)
            }
            ButtonView(button: .dPadDown)
                .padding(.horizontal)
        }
        .frame(width: size, height: size)
        .onAppear() {
            if UIDevice.current.systemName.contains("iPadOS") {
                size *= 1.2
            }
            
            size *= CGFloat(controllerScale)
        }
    }
}

struct ABXYView: View {
    @State var size: CGFloat = 145
    @AppStorage("On-ScreenControllerScale") var controllerScale: Double = 1.0
    
    /*var body: some View {
        VStack {
            ButtonView(button: .X)
            HStack {
                ButtonView(button: .Y)
                Spacer(minLength: 20)
                ButtonView(button: .A)
            }
            ButtonView(button: .B)
                .padding(.horizontal)
        }
    */
    //拳皇新布局
    var body: some View {
        VStack(spacing: 5){
            HStack(spacing: 5) {
                ButtonView(button: .Y)
                ButtonView(button: .B)
                   
            }
            HStack(spacing: 5) {
                ButtonView(button: .X)
                ButtonView(button: .A)
                   
            }
        }
        .frame(width: size, height: size)
        .onAppear() {
            if UIDevice.current.systemName.contains("iPadOS") {
                size *= 1.2
            }
            
            size *= CGFloat(controllerScale)
        }
    }
}

struct ButtonView: View {
    var button: VirtualControllerButton
    @State var width: CGFloat = 90
    @State var height: CGFloat = 90
    @State var isPressed = false
    @AppStorage("onscreenhandheld") var onscreenjoy: Bool = false
    @Environment(\.colorScheme) var colorScheme
    @Environment(\.presentationMode) var presentationMode
    @AppStorage("On-ScreenControllerScale") var controllerScale: Double = 1.0
    

    
    var body: some View {
        Image(systemName: buttonText)
            .resizable()
            .frame(width: width, height: height)
            .foregroundColor(colorScheme == .dark ? Color.gray : Color.gray)
            .opacity(isPressed ? 0.3 : 0.6)
            .gesture(
                DragGesture(minimumDistance: 0)
                    .onChanged { _ in
                        if !self.isPressed {
                            self.isPressed = true
                            Ryujinx.shared.virtualController.setButtonState(1, for: button)
                            Haptics.shared.play(.light)
                        }
                    }
                    .onEnded { _ in
                        self.isPressed = false
                        Ryujinx.shared.virtualController.setButtonState(0, for: button)
                    }
                )
            .onAppear() {
                if button == .leftTrigger || button == .rightTrigger || button == .leftShoulder || button == .rightShoulder {
                    width = 65
                }
            
                
                if button == .back || button == .start || button == .guide {
                    width = 35
                    height = 35
                }
                
                if UIDevice.current.systemName.contains("iPadOS") {
                    width *= 1.2
                    height *= 1.2
                }
                
                width *= CGFloat(controllerScale)
                height *= CGFloat(controllerScale)
            }
    }
    

    
    private var buttonText: String {
        switch button {
        case .A:
            return "a.circle.fill"
        case .B:
            return "b.circle.fill"
        case .X:
            return "x.circle.fill"
        case .Y:
            return "y.circle.fill"
        case .dPadUp:
            return "arrowtriangle.up.circle.fill"
        case .dPadDown:
            return "arrowtriangle.down.circle.fill"
        case .dPadLeft:
            return "arrowtriangle.left.circle.fill"
        case .dPadRight:
            return "arrowtriangle.right.circle.fill"
        case .leftTrigger:
            return"zl.rectangle.roundedtop.fill"
        case .rightTrigger:
            return "zr.rectangle.roundedtop.fill"
        case .leftShoulder:
            return "l.rectangle.roundedbottom.fill"
        case .rightShoulder:
            return "r.rectangle.roundedbottom.fill"
        case .start:
            return "plus.circle.fill" // System symbol for +
        case .back:
            return "minus.circle.fill" // System symbol for -
        case .guide:
            return "house.circle.fill"
        // This should be all the cases
        default:
            return ""
        }
    }
}


